// cotroller.hh - controller interface
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CONTROLLER_HH
#define CONTROLLER_HH

namespace control {

    // abstract base class controller
    // 
    // defines numbers for active controls:
    //    bup, bdown, bleft, bright,
    //    bfire1, bfire2, bfire3, bfire4,
    //    bcont1, bcont2, bcont3, bcont4.
    // protected member:
    //    action: contains the status of the controller (which controls
    //            are active)
    // pure virtual funciton:
    //    poll(): polls the state and updates action
    // other functions:
    //    get():  returns a logic OR of all the active controls
    //    up(), down(), ..., bcont4(): true if the respective control is
    //                                 active
    class controller {
    public:
        enum {
            bup = 0x0001,
            bdown = 0x0002,
            bleft = 0x0004,
            bright = 0x0008,
            bfire1 = 0x0010,
            bfire2 = 0x0020,
            bfire3 = 0x0040,
            bfire4 = 0x0080,
            bcont1 = 0x0100,
            bcont2 = 0x0200,
            bcont3 = 0x0400,
            bcont4 = 0x0800
        };

        controller()
            : action(0) { }
        virtual ~controller() { }
        virtual void poll() = 0;
        
        unsigned get() const { return action; }
        bool up() const      { return action & bup; }
        bool down() const    { return action & bdown; }
        bool left() const    { return action & bleft; }
        bool right() const   { return action & bright; }
        bool fire1() const   { return action & bfire1; }
        bool fire2() const   { return action & bfire2; }
        bool fire3() const   { return action & bfire3; }
        bool fire4() const   { return action & bfire4; }
        bool cont1() const   { return action & bcont1; }
        bool cont2() const   { return action & bcont2; }
        bool cont3() const   { return action & bcont3; }
        bool cont4() const   { return action & bcont4; }

    protected:
        unsigned action;
    };

} // namespace control

#endif // ~CONTROLLER_HH
