// main.cc - main()
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <ctime>
#include <exception>
#include <fstream>
#include <iostream>
#include <string>
#include "SDL.h"
#include "SDL_image.h"

#include "game.hh"
#include "misc.hh"
#include "record.hh"
#include "sdl_controller.hh"
#include "sdl_graphics.hh"
#include "sdl_timer.hh"
#include <unistd.h>
#include <iostream.h>

extern "C" int main(int argc, char* argv[])
{
    try {
        // load options and ball data
        rec::recrecord options;
        {
            std::ifstream is("data/options.dat");
            is >> options;
        }
        rec::recrecord bd;
        {
            std::string filename(options["game"]["ball_data"]);
            std::ifstream is(filename.c_str());
            is >> bd;
        }

#ifdef __BEOS__
int counti;
char datapath[256];
for (counti=strlen(argv[0]) ; argv[0][counti]!='/' ; counti--);
strncpy(datapath, argv[0], counti);
chdir(datapath);
#endif

        // initialise SDL
        SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
        SDL_ShowCursor(0);

        // load icon, convert it to 32 bit, and set as icon
        std::string icon_filename = options["graphics"]["icon"];
        SDL_Surface* icon = IMG_Load(icon_filename.c_str());
        SDL_Surface* icon32 = SDL_CreateRGBSurface(0, icon->w, icon->h, 32,
            0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
        SDL_Surface* the_icon = SDL_ConvertSurface(icon, icon32->format, 0);
        SDL_SetColorKey(the_icon, SDL_SRCCOLORKEY,
                        *static_cast<Uint32*>(the_icon->pixels));
        SDL_WM_SetIcon(the_icon, 0);
        SDL_FreeSurface(the_icon);
        SDL_FreeSurface(icon32);
        SDL_FreeSurface(icon);

        SDL_WM_SetCaption("Tunnel", "tunnel");

        rec::record gro = options["graphics"];
        SDL_Surface* s = SDL_SetVideoMode
            (gro["width"], gro["height"], gro["depth"],
             (gro["anyformat"] ? SDL_ANYFORMAT : 0)
             | (gro["fullscreen"] ? SDL_FULLSCREEN : 0) | SDL_DOUBLEBUF);
        
        sdl::tsurface* ts = new sdl::tsurface(s);
        sdl::controller* c = new sdl::controller;
        sdl::timer* t = new sdl::timer;
        misc::random r(std::time(0));
        
        Game::game g(options["graphics"]["width"],
                     options["graphics"]["height"],
                     options["game"], ts, &bd, c, t, r);

        for (bool ready = false; !ready; ) {
            g.play();
            if (g.stat() == Game::game::pause) {
                // make sure player releases ESC
                while (c -> cont1()) {
                    SDL_WaitEvent(0);
                    c->poll();
                }
                for (bool ready2 = false; !ready2; ) {
                    SDL_WaitEvent(0);
                    c->poll();
                    if (c -> cont1())
                        ready = ready2 = true;
                    if (c -> cont2())
                        ready2 = true;
                }
            }
            else
                ready = true;
        }
        
        SDL_Quit();
    }
    catch (std::exception& e) {
        std::cerr << e.what() << endl;
    }
    catch (...) {
        std::cerr << "Caught an exception." << endl;
    }
    return 0;
}
