// misc.c - contains code required by some miscellaneous classes
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "misc.hh"

// give a random number.
// Function borrowed from rand_r() in glibc-2.1.2 (C) FSF,
// written by Ulrich Drepper
int misc::random::operator()()
{
    int result;
    
    seed *= 1103515245;
    seed += 12345;
    result = (unsigned int) (seed / 65536) % 2048;

    seed *= 1103515245;
    seed += 12345;
    result <<= 10;
    result ^= (unsigned int) (seed / 65536) % 1024;

    seed *= 1103515245;
    seed += 12345;
    result <<= 10;
    result ^= (unsigned int) (seed / 65536) % 1024;

    return result;
}

int misc::random::max() const
{
    return 0x7fffffff;
}

// set up a lookup table
void misc::scaler1::set(int src, int dst)
{
    // First build lookup table for x
    map = new int[dst];
    
    // We split each dst pixel into (srcw * 2) parts and each src pixel
    // into (dstw * 2) parts, and superimpose the images, which both have
    // (srcw * dstw * 2) parts. Then for each dst pixel, take its middle
    // part (for 1st pixel:srcw, for 2nd pixel 3*srcw, for 3rd 5*srcw...)
    // and find on which pixel in src it lies on.
    int mid, dmid, dvdr;
    
    mid = src;
    dmid = mid * 2;
    dvdr = dst * 2;
    for (int i = 0; i < dst; ++i) {
        map[i] = mid / dvdr;
        mid += dmid;
    }
}

void misc::scaler1::clear()
{
    delete[] map;
}

// set up a lookup table
void misc::scaler::set(int srcw, int srch, int dstw, int dsth)
{
    sw = srcw;
    sh = srch;
    dw = dstw;
    dh = dsth;
    // First build lookup table for x
    mapx = new int[dstw];
    
    // We split each dst pixel into (srcw * 2) parts and each src pixel
    // into (dstw * 2) parts, and superimpose the images, which both have
    // (srcw * dstw * 2) parts. Then for each dst pixel, take its middle
    // part (for 1st pixel:srcw, for 2nd pixel 3*srcw, for 3rd 5*srcw...)
    // and find on which pixel in src it lies on.
    int mid, dmid, dvdr;
    
    mid = srcw;
    dmid = mid * 2;
    dvdr = dstw * 2;
    for (int x = 0; x < dstw; ++x) {
        mapx[x] = mid / dvdr;
        mid += dmid;
    }
    
    // See if we can do with only one lookup table, otherwise repeat process
    // for y map.
    if (srcw == srch && dstw == dsth) {
        mapy = mapx;
    }
    else {
        mapy = new int[dsth];
        
        mid = srch;
        dmid = mid * 2;
        dvdr = dsth * 2;
        for (int y = 0; y < dsth; ++y) {
            mapy[y] = mid / dvdr;
            mid += dmid;
        }
    }
}

void misc::scaler::clear()
{
    // If we use only one lookup table, delete only once.
    if (mapy != mapx)
        delete[] mapy;
    delete[] mapx;
}
