// new_ball.cc - to create new balls
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <string>
#include <vector>

#include "ball.hh"
#include "misc.hh"
#include "new_ball.hh"

void balls::def_creator::period::add(const std::string& n, int c)
{
    m[n] += c;
}

void balls::def_creator::period::remove(std::string& n)
{
    map::iterator i = m.find(n);
    if (i != m.end())
        m.erase(i);
}

void balls::def_creator::period::prepare()
{
    int size = m.size();
    name.clear();
    name.reserve(size);
    std::vector<int> chance;
    chance.reserve(size);
    for (map::iterator i = m.begin(); i != m.end(); ++i) {
        name.push_back(i->first);
        chance.push_back(i->second);
    }
    ran.reset(chance.begin(), chance.end());
}

const std::string& balls::def_creator::period::get(misc::random& r)
{
    return name[ran(r)];
}

balls::def_creator::def_creator(ball_info* inf, ship* s)
    : info(inf),
      sp(s),
      time(0),
      new_ball_calls(0),
      av_vel(-1.0)
{
    period p;
    p.add("none", 90);
    p.add("normal_ball", 10);
    periods.push_back(p);
    periods.back().prepare();
    p.add("none", 90);
    p.add("hard_ball", 10);
    periods.push_back(p);
    periods.back().prepare();
    p.add("none", 90);
    p.add("fast_ball", 10);
    periods.push_back(p);
    periods.back().prepare();
    p.add("none", 90);
    p.add("big_ball", 10);
    periods.push_back(p);
    periods.back().prepare();
    p.add("none", 90);
    p.add("whiz_ball", 10);
    periods.push_back(p);
    periods.back().prepare();
}

void balls::def_creator::move(int t)
{
    time += t;

    // We have to see if we are to add something new.
    // If tunnel is stationary, we need a ball for every x units:
    //   req = time / x
    // If tunnel is moving, we want the same concentration per unit
    // tunnel length. We map depth to equivalent time by
    //   time_eq = depth / velocity
    //     where depth is the depth moved and
    //           velocity is the average ball velocity
    // So now
    //   req = (time + depth / velocity) / x
    // A ball can be thrown every 50 time units.
    double depth = sp->loc().z();
    for (int req = int((time + depth / -av_vel) / 50);
         new_ball_calls < req;
         ++new_ball_calls) {
        misc::random& rand = *info->rand;
        int i = new_ball_calls >= 500 ? 4 : new_ball_calls / 100;
        const std::string& n = periods[i].get(rand);
        if (n != "none") {
            rec::record& r = (*info->bd)[n];
            vect3d<> ll(misc::rndpolar(rand, tunnel_r - double(r["radius"])),
                        depth + tunnel_dpth);
            vect3d<> vv(misc::rndpm(rand, double(r["velxy"])),
                        misc::rndpm(rand, double(r["velxy"])),
                        av_vel + misc::rndpm(rand, double(r["velz"])));
            ball* b = name_to_nbf(n)(ll, vv, info);
            info->swal -> swallow(b);
        }
    }
}
