// record.cc - storing of data
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <iostream>
#include <strstream> // because of no stringstream
#include <string>

#include "record.hh"
#include "except.hh"

// read from an istream to an item, setting the failbit of the istream
// on failure
std::istream& rec::operator>>(std::istream& is, item& i)
{
    try {
        // read the type, and the value accordingly
        std::string s;
        if (!(is >> s))
            throw fail();

        if (s == "bool") {
            bool b;
            if (!(is >> b))
                throw fail();
            i = b;
        }
        else if (s == "int") {
            int j;
            if (!(is >> j))
                throw fail();
            i = j;
        }
        else if (s == "double") {
            double d;
            if (!(is >> d))
                throw fail();
            i = d;
        }
        else if (s == "string") {
            if (!(is >> s))
                throw fail();
            i = s;
        }
        else if (s == "record") {
            record rr;
            if (!(is >> rr))
                throw fail();
            i = rr;
        }
        else if (s == "recrecord") {
            recrecord rr;
            if (!(is >> rr))
                throw fail();
            i = rr;
        }
        else if (s == "unknown") {
            if (!(is >> s))
                throw fail();
            i = item();
        }
        else
            throw fail();
    }
    catch(fail) {
        is.setstate(std::ios::failbit);
        return is;
    }
    return is;
}

// write the item to an ostream
std::ostream& rec::operator<<(std::ostream& os, const item& i)
{
    if (i.is_type(tag<bool>()))
        os << "bool " << i.ref(tag<bool>());
    else if (i.is_type(tag<std::string>()))
        os << "string " << i.ref(tag<std::string>());
    else if (i.is_type(tag<double>()))
        os << "double " << i.ref(tag<double>());
    else if (i.is_type(tag<int>()))
        os << "int " << i.ref(tag<int>());
    else if (i.is_type(tag<record>()))
        os << "record " << i.ref(tag<record>());
    else if (i.is_type(tag<recrecord>()))
        os << "recrecord " << i.ref(tag<recrecord>());
    else
        os << "unknown x";
    return os;
}
