// sdl_controller.cc - controller functions using SDL
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "SDL.h"

#include "sdl_controller.hh"

// only passes on key events.
int sdl::filter_events(const SDL_Event* event)
{
    switch (event->type) {
        case SDL_KEYDOWN:
            return 1;
        case SDL_KEYUP:
            return 1;
        default:
            ;
    }
    return 0;
}

sdl::controller::controller()
{
    SDL_SetEventFilter(filter_events);
}

sdl::controller::~controller()
{
}

// update action according to which keys have been pressed/released.
// On pressing of keys specific to bcont1, bcont2, bcont3 and bcont4,
// call the respective handlers.
void sdl::controller::poll()
{
    SDL_Event event;
    while (SDL_PollEvent(&event)) {
        switch (event.type) {
            case SDL_KEYDOWN:
                switch (event.key.keysym.sym) {
                    case SDLK_UP:
                        action |= bup; break;
                    case SDLK_DOWN:
                        action |= bdown; break;
                    case SDLK_LEFT:
                        action |= bleft; break;
                    case SDLK_RIGHT:
                        action |= bright; break;
                    case SDLK_z:
                        action |= bfire1; break;
                    case SDLK_x:
                        action |= bfire2; break;
                    case SDLK_c:
                        action |= bfire3; break;
                    case SDLK_s:
                        action |= bfire4; break;
                    case SDLK_ESCAPE:
                        action |= bcont1; break;
                    case SDLK_F1:
                        action |= bcont2; break;
                    case SDLK_F2:
                        action |= bcont3; break;
                    case SDLK_F3:
                        action |= bcont4; break;
                    default:
                        ;
                }
                break;
            case SDL_KEYUP:
                switch (event.key.keysym.sym) {
                    case SDLK_UP:
                        action &= ~bup; break;
                    case SDLK_DOWN:
                        action &= ~bdown; break;
                    case SDLK_LEFT:
                        action &= ~bleft; break;
                    case SDLK_RIGHT:
                        action &= ~bright; break;
                    case SDLK_z:
                        action &= ~bfire1; break;
                    case SDLK_x:
                        action &= ~bfire2; break;
                    case SDLK_c:
                        action &= ~bfire3; break;
                    case SDLK_s:
                        action &= ~bfire4; break;
                    case SDLK_ESCAPE:
                        action &= ~bcont1; break;
                    case SDLK_F1:
                        action &= ~bcont2; break;
                    case SDLK_F2:
                        action &= ~bcont3; break;
                    case SDLK_F3:
                        action &= ~bcont4; break;
                    default:
                        ;
                }
                break;
            default:
                break;
        }
    }
}
