// timer.hh - provides the timer interface
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef TIMER_HH
#define TIMER_HH

namespace Timer {

    // abstract base class timer:
    // set_delay: set the delay per tick. Should be able to handle
    //            calls to this also when the timer is already running.
    //            In this case, the new delay comes into effect as from
    //            last tick.
    // get_delay: ask what the delay per tick is
    // start:     start the timer ticking and set the ticks to zero. If
    //            called when timer is already running, just set ticks.
    // stop:      stop the timer ticking
    // wait:      wait until there is at least one tick
    // ticks:     inquire number of ticks
    // done:      reduce one tick

    class timer {
    public:
        virtual ~timer() {}
        virtual void set_delay(int msec) = 0;
        virtual int get_delay() = 0;
        virtual void start() = 0;
        virtual void stop() = 0;
        virtual void wait() = 0;
        virtual void done() = 0;
        virtual int ticks() = 0;
    };
} // namespace Timer

#endif // TIMER_HH
